﻿using System;
using System.Runtime.Remoting.Messaging;
using System.Collections.Generic;
using System.Xml;
using System.Reflection;

namespace Earthworm
{
    /// <summary>
    /// 方面管理器
    /// </summary>
    internal class Aspect : IMessageSink,IDisposable
    {
        /// <summary>
        /// 消息槽
        /// </summary>
        private IMessageSink sink;
        /// <summary>
        /// 构造函数
        /// </summary>
        /// <param name="sink">消息槽</param>
        /// <param name="assemblyName">处理切面对象的程序集</param>
        /// <param name="className">处理切面对象的全名</param>
        internal Aspect(IMessageSink sink, string assemblyName, string className)
        {
            this.sink = sink;
            IAdviceProvider provider = null;
            try
            {
                provider = Assembly.LoadFrom(assemblyName).CreateInstance(className) as IAdviceProvider;
                beforeAdvices = provider.BeforeAdvices;
                stopAdvices = provider.StopAdvices;
                afterAdvices = provider.AfterAdvices;
            }
            catch{}
        }
        /// <summary>
        /// 异步处理给定的消息，太麻烦，没做
        /// </summary>
        /// <param name="msg">要处理的消息</param>
        /// <param name="replySink">答复消息的答复接收器</param>
        /// <returns>提供一种在调度异步消息之后控制这些消息的方法的对象</returns>
        public IMessageCtrl AsyncProcessMessage(IMessage msg, IMessageSink replySink)
        {
            return null;
        }
        /// <summary>
        /// 获取接收器链中的下一个消息接收器
        /// </summary>
        public IMessageSink NextSink
        {
            get
            {
                return sink;
            }
        }
        /// <summary>
        /// 同步处理给定的消息
        /// </summary>
        /// <param name="msg">要处理的消息</param>
        /// <returns>响应请求的答复消息</returns>
        public IMessage SyncProcessMessage(IMessage msg)
        {
            IMethodCallMessage call = msg as IMethodCallMessage;
            string methodName = call.MethodName;
            IBeforeAdvice before = FindAdvice(methodName, beforeAdvices);
            if (before != null)
            {
                before.DoAdvice(call);
            }
            IStopAdvice stop = FindAdvice(methodName, stopAdvices);
            IMessage retMsg = null;
            if (stop != null)
            {
                if (stop.CanContinueMethod(call))
                {
                    retMsg = NextSink.SyncProcessMessage(msg);
                }
            }
            else
            {
                retMsg = NextSink.SyncProcessMessage(msg);
            }
            if (retMsg!=null)
            {
                IMethodReturnMessage reply = retMsg as IMethodReturnMessage;
                IAfterAdvice after = FindAdvice(methodName, afterAdvices);
                if (after != null)
                {
                    after.DoAdvice(reply);
                }
                return retMsg;
            }
            return new ReturnMessage(null, call);

        }
        /// <summary>
        /// 前通知集合
        /// </summary>
        private Dictionary<string, IBeforeAdvice> beforeAdvices ;
        /// <summary>
        /// 拦截通知集合
        /// </summary>
        private  Dictionary<string, IStopAdvice> stopAdvices ;
        /// <summary>
        /// 后通知集合
        /// </summary>
        private  Dictionary<string, IAfterAdvice> afterAdvices;
        /// <summary>
        /// 在通知集合中找到方法名name的通知
        /// </summary>
        /// <typeparam name="T">泛型</typeparam>
        /// <param name="name">方法名</param>
        /// <param name="advices">通知集合</param>
        /// <returns>泛型对象</returns>
        private T FindAdvice<T>(string name, Dictionary<string, T> advices)
        {
            T ret = default(T);
            if (advices==null)
            {
                return ret;
            }
            lock (advices)
            {
                if (advices.ContainsKey(name))
                {
                    return advices[name];
                }
            }
            return ret;
        }
        /// <summary>
        /// 囧
        /// </summary>
        ~Aspect()
        {
            Dispose();
        }
        /// <summary>
        /// 囧囧囧囧囧囧囧囧囧
        /// </summary>
        public void Dispose()
        {
            sink = null;
            GC.SuppressFinalize(this);
        }
    }
}
