﻿using System;
using System.Runtime.Remoting.Contexts;
using System.Runtime.Remoting.Activation;

namespace Earthworm
{
    /// <summary>
    /// 面向切面属性
    /// 用来配置处理切面的对象
    /// </summary>
    [AttributeUsage(AttributeTargets.Class)]
    public sealed class AspectOrientedAttribute:Attribute,IContextAttribute,IDisposable
    {
        /// <summary>
        /// 处理切面对象的程序集
        /// </summary>
        private string assemblyName;
        /// <summary>
        /// 处理切面对象的全名
        /// </summary>
        private string className;
        /// <summary>
        /// 构造函数
        /// </summary>
        /// <param name="assemblyName">处理切面对象的程序集</param>
        /// <param name="className">处理切面对象的全名</param>
        public AspectOrientedAttribute(string assemblyName, string className)
        {
            this.assemblyName = assemblyName;
            this.className = className;
        }
        /// <summary>
        /// 囧
        /// </summary>
        ~AspectOrientedAttribute()
        {
            Dispose();
        }
        /// <summary>
        /// 囧囧囧囧囧囧囧囧囧囧囧囧囧囧囧
        /// </summary>
        public void Dispose()
        {
            assemblyName = null;
            className = null;
            GC.SuppressFinalize(this);
        }
        /// <summary>
        /// 在给定消息中将上下文属性返回给调用方
        /// </summary>
        /// <param name="msg">将上下文属性添加到的 消息</param>
        public void GetPropertiesForNewContext(IConstructionCallMessage msg)
        {
            msg.ContextProperties.Add(new AspectOrientedProperty(assemblyName,className));
        }
        /// <summary>
        /// 返回一个布尔值，指示指定上下文是否满足上下文属性的要求
        /// </summary>
        /// <param name="ctx">当前上下文属性检查所依据的上下文</param>
        /// <param name="msg">构造调用，需要依据当前上下文检查其参数</param>
        /// <returns>如果传入的上下文一切正常，则为 true；否则为 false。给false</returns>
        public bool IsContextOK(Context ctx, IConstructionCallMessage msg)
        {
            return false;
        }
    }
}
