﻿using System.Collections.Generic;

namespace Earthworm
{
    /// <summary>
    /// 通知提供者接口
    /// 实现此接口的对象必须包含无参的构造函数
    /// 否则将无效
    /// 以下所有设置的方法名必须是非static方法的方法名称
    /// </summary>
    public interface IAdviceProvider 
    {
        /// <summary>
        /// 前通知集合，其中key为要进行前通知的方法名，value为进行前通知时的处理对象
        /// </summary>
        Dictionary<string, IBeforeAdvice> BeforeAdvices { get; }
        /// <summary>
        /// 拦截通知集合，其中key为要拦截方法名，value为进行拦截时的处理对象
        /// </summary>
        Dictionary<string, IStopAdvice> StopAdvices { get; }
        /// <summary>
        /// 后通知集合，其中key为要进行后通知的方法名，value为进行后通知时的处理对象
        /// </summary>
        Dictionary<string, IAfterAdvice> AfterAdvices { get; }
    }
    //////////////////////////////////////////////////////////////////////////
    /**示例
    /// <summary>
    /// 日志通知提供对象
    /// </summary>
    class LoggerProvider:IAdviceProvider
    {
        private Dictionary<string, IBeforeAdvice> beforeAdvices = new Dictionary<string, IBeforeAdvice>();
        private Dictionary<string, IAfterAdvice> afterAdvices = new Dictionary<string, IAfterAdvice>();
        /// <summary>
        /// 必须包含无参构造函数
        /// </summary>
        public LoggerProvider()
        {
            LoggerAdvice la = new LoggerAdvice();
            //将SetCurrectUser作为前通知方法
            beforeAdvices.Add("SetCurrectUser", la);
            //将Logout作为后通知方法
            afterAdvices.Add("Logout", la);
        }
        /// <summary>
        /// 前通知集合
        /// </summary>
        public Dictionary<string, IBeforeAdvice> BeforeAdvices
        {
            get { return beforeAdvices; }
        }
        /// <summary>
        /// 拦截通知集合，与日志无关，不返回任何信息
        /// </summary>
        public Dictionary<string, IStopAdvice> StopAdvices
        {
            get { return null; }
        }
        /// <summary>
        /// 后通知集合
        /// </summary>
        public Dictionary<string, IAfterAdvice> AfterAdvices
        {
            get { return afterAdvices; }
        }
    }
*/
    //////////////////////////////////////////////////////////////////////////
}
