﻿using System.Runtime.Remoting.Messaging;

namespace Earthworm
{
    /// <summary>
    /// 拦截通知接口
    /// </summary>
    public interface IStopAdvice
    {
        /// <summary>
        /// 是否可以继续方法的执行
        /// </summary>
        /// <param name="msg">要执行方法的调用消息</param>
        /// <returns>如果可以继续执行，返回true，否则为false</returns>
        bool CanContinueMethod(IMethodCallMessage msg);
    }
    //////////////////////////////////////////////////////////////////////////
    /**示例
    /// <summary>
    /// 检查对象权限，可以在用户进行关键操作的方法前
    /// 比如btn_DeleteUser_Click方法
    /// </summary>
    class CheckPermission : IStopAdvice
    {
        /// <summary>
        /// 检查当前用户的权限
        /// 如果是管理员则可以继续执行方法
        /// 否则不能执行
        /// </summary>
        /// <param name="msg">要执行方法的调用消息</param>
        /// <returns>是管理员返回true,否则为false</returns>
        public bool CanContinueMethod(IMethodCallMessage msg)
        {
            if (User.CurrentUser.IsAdministrator)
                return true;
            else
                return false;
        }
    }
     * */
    //////////////////////////////////////////////////////////////////////////
}
